==> libreswan_5.2.orig.tar.gz: libreswan-5.2/lib/libswan/ttoselectors_num.c <==
/* cloning an selector list, for libreswan
 *
 * Copyright (C) 2022 Andrew Cagney
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <https://www.gnu.org/licenses/gpl2.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <stdarg.h>

#include "lswalloc.h"
#include "passert.h"
#include "lswlog.h"		/* for dbg() */
#include "ip_selector.h"
#include "ip_info.h"

#define ip_token ip_selector
#define ip_tokens ip_selectors

diag_t ttoselectors_num(shunk_t input, const char *delims,
			const struct ip_info *input_afi,
			ip_tokens *output,
			ip_address *nonzero_host)
{
	zero(output);
	*nonzero_host = unset_address;

	if (input.ptr == NULL) {
		return NULL;
	}

	dbg("%s() input: "PRI_SHUNK, __func__, pri_shunk(input));

	/*
	 * Two passes:
	 *
	 *   Pass 1: determine the number of tokens.
	 *   Pass 1.5: allocate list pointers; zero counters
	 *   Pass 2: save the values in separate IPv[46] lists.
	 */

	struct shunks *tokens = shunks(input, delims, KEEP_EMPTY_SHUNKS, HERE); /* must free */
	if (tokens->kept_empty_shunks) {
		pfree(tokens);
		return diag("empty field");
	}

	for (unsigned pass = 1; pass <= 2; pass++) {
		unsigned nr_tokens = 0;
		FOR_EACH_ITEM(token, tokens) {
			passert(token->len > 0);
			ip_token tmp_token;
			ip_address tmp_nonzero;
			err_t e = ttoselector_num(*token, input_afi, &tmp_token, &tmp_nonzero);
			const struct ip_info *afi = selector_info(tmp_token);
			switch (pass) {
			case 1:
				/* validate during first pass */
				if (e != NULL) {
					diag_t d = diag(PRI_SHUNK" invalid, %s",
							pri_shunk(*token), e);
					pfree(tokens);
					return d;
				}
				if (tmp_nonzero.is_set && !nonzero_host->is_set) {
					*nonzero_host = tmp_nonzero; /* save first */
				}
				break;
			case 2:
				/* save value during second pass */
				passert(e == NULL);
				output->ip[afi->ip_index].list[output->ip[afi->ip_index].len] = tmp_token;
				break;
			}
			/* advance the lengths */
			output->ip[afi->ip_index].len++;
			nr_tokens++;
		}
		if (nr_tokens == 0) {
			pfree(tokens);
			return NULL;
		}
		switch (pass) {
		case 1:
			/*
			 * Pass 1.5: Allocate.
			 */
			dbg("%s() nr tokens %u", __func__, nr_tokens);
			output->list = alloc_things(ip_token, nr_tokens, "selectors");
			FOR_EACH_ELEMENT(afi, ip_families) {
				enum ip_index ip = afi->ip_index;
				output->ip[ip].list = output->list + output->len;
				output->len += output->ip[ip].len;
				output->ip[ip].len = 0; /* ready for second pass */
			}
			passert(output->len == nr_tokens);
			break;
		case 2:
			passert(nr_tokens == output->len);
			break;
		}
	}

	pfree(tokens);
	return NULL;
}

==> libreswan-5.3.tar.gz: libreswan-5.3/lib/libswan/ttoselectors_num.c <==
/* cloning an selector list, for libreswan
 *
 * Copyright (C) 2022 Andrew Cagney
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <https://www.gnu.org/licenses/gpl2.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <stdarg.h>

#include "lswalloc.h"
#include "passert.h"
#include "lswlog.h"		/* for dbg() */
#include "ip_selector.h"
#include "ip_info.h"

#define ip_token ip_selector
#define ip_tokens ip_selectors

diag_t ttoselectors_num(shunk_t input, const char *delims,
			const struct ip_info *input_afi,
			ip_tokens *output,
			ip_address *nonzero_host)
{
	zero(output);
	*nonzero_host = unset_address;

	if (input.ptr == NULL) {
		return NULL;
	}

	dbg("%s() input: "PRI_SHUNK, __func__, pri_shunk(input));

	/*
	 * Two passes:
	 *
	 *   Pass 1: determine the number of tokens.
	 *   Pass 1.5: allocate list pointers; zero counters
	 *   Pass 2: save the values in separate IPv[46] lists.
	 */

	struct shunks *tokens = ttoshunks(input, delims, KEEP_EMPTY_SHUNKS); /* must free */

	if (tokens->kept_empty_shunks) {
		pfree(tokens);
		return diag("empty field");
	}

	if (tokens->len == 0) {
		pfree(tokens);
		return NULL;
	}

	dbg("%s() nr tokens %u", __func__, tokens->len);

	output->list = alloc_things(ip_token, tokens->len, "selectors");
	output->len = tokens->len;

	unsigned nr = 0;
	ITEMS_FOR_EACH(token, tokens) {
		ip_address tmp_nonzero;
		err_t e = ttoselector_num(*token, input_afi,
					  &output->list[nr++],
					  &tmp_nonzero);
		/* validate during first pass */
		if (e != NULL) {
			diag_t d = diag(PRI_SHUNK" invalid, %s",
					pri_shunk(*token), e);
			pfree(tokens);
			pfree(output->list);
			zero(output);
			return d;
		}
		if (tmp_nonzero.ip.is_set && !nonzero_host->ip.is_set) {
			*nonzero_host = tmp_nonzero; /* save first */
		}
	}

	passert(output->len == tokens->len);
	pfree(tokens);
	return NULL;
}
